"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const observable_1 = require("@reactive-js/observable");
const pipe_1 = require("@reactive-js/pipe");
const identity_1 = require("./identity");
const lift_1 = require("./lift");
const createAcc = (enumerator) => {
    const onNotifyDispatch = (_) => {
        enumerator.dispatch();
    };
    return pipe_1.pipe(identity_1.identity(), lift_1.lift(observable_1.onNotify(onNotifyDispatch)));
};
const createResources = (enumerator) => (scheduler) => [
    createAcc(enumerator).enumerateAsync(scheduler),
    identity_1.identity().enumerateAsync(scheduler),
];
const createFactory = (withLatestFrom, initial, enumerator) => (request, src) => {
    const mapReducerRequestToAcc = observable_1.map(({ acc }) => acc);
    const doOnDispose = (_) => {
        src.dispose();
    };
    const setupOnSubscribe = () => {
        const next = pipe_1.pipe(enumerator, observable_1.onDispose(doOnDispose));
        src.dispatch(next);
    };
    const notifySrc = (next) => {
        if (next.type === 1) {
            request.dispatch(next);
        }
        else {
            src.dispatch(observable_1.empty());
        }
    };
    return pipe_1.pipe(observable_1.merge(observable_1.compute(() => ({
        type: 1,
        acc: initial(),
    })), pipe_1.pipe(src, observable_1.switchAll(), withLatestFrom(mapReducerRequestToAcc(request)))), observable_1.onNotify(notifySrc), mapReducerRequestToAcc, observable_1.takeLast(), observable_1.onSubscribe(setupOnSubscribe));
};
const consumeImpl = (withLatestFrom, initial) => enumerator => observable_1.using(createResources(enumerator), createFactory(withLatestFrom, initial, enumerator));
const consume = (reducer, initial) => {
    const withLatestSelector = (next, acc) => reducer(acc, next);
    return consumeImpl(acc => observable_1.withLatestFrom(acc, withLatestSelector), initial);
};
const consumeAsync = (reducer, initial) => {
    const withLatestSelector = (next, acc) => pipe_1.pipe(reducer(acc, next), observable_1.takeFirst());
    return consumeImpl(acc => pipe_1.compose(observable_1.withLatestFrom(acc, withLatestSelector), observable_1.switchAll()), initial);
};
exports.reduce = (reducer, initial) => enumerable => observable_1.using(scheduler => enumerable.enumerateAsync(scheduler), consume(reducer, initial));
exports.reduceAsync = (reducer, initial) => enumerable => observable_1.using(scheduler => enumerable.enumerateAsync(scheduler), consumeAsync(reducer, initial));
//# sourceMappingURL=reduce.js.map