import { generate as generateObs, map as mapObs, keepType, takeFirst, never, concat, scanAsync, empty, genMap, createObservable, subscribe, onNotify, ofValue, } from "@reactive-js/observable";
import { none, isSome } from "@reactive-js/option";
import { compose, pipe } from "@reactive-js/pipe";
import { createAsyncEnumerable } from "./createAsyncEnumerable";
import { map } from "./map";
import { toPausableScheduler } from "@reactive-js/scheduler";
const createStream = (f) => createAsyncEnumerable(obs => concat(f(obs), never()));
const emptyModeMapper = (mode) => mode === 1 ? { type: 2 } : none;
const onEmptyOperator = compose(mapObs(emptyModeMapper), keepType(isSome), takeFirst());
export const emptyStream = () => createStream(onEmptyOperator);
const ofValueOperator = (data) => genMap(function* (mode) {
    switch (mode) {
        case 1:
            yield { type: 1, data };
            yield { type: 2 };
    }
});
export const ofValueStream = (value) => createStream(ofValueOperator(value));
const generateScanner = (generator, delay) => (acc, ev) => ev === 1 ? generateObs(generator, () => acc, delay) : empty();
export const generateStream = (generator, initialValue, delay = 0) => createStream(compose(scanAsync(generateScanner(generator, delay), initialValue, 1), mapObs(data => ({ type: 1, data }))));
export const mapStream = (mapper) => map((ev) => ev.type === 1
    ? {
        type: 1,
        data: mapper(ev.data),
    }
    : { type: 2 });
export const fromObservableStream = (observable) => createAsyncEnumerable(modeObs => createObservable(subscriber => {
    const pausableScheduler = toPausableScheduler(subscriber);
    const modeSubscription = pipe(modeObs, onNotify(mode => {
        switch (mode) {
            case 2:
                pausableScheduler.pause();
                break;
            case 1:
                pausableScheduler.resume();
                break;
        }
    }), subscribe(subscriber));
    const eventStream = concat(pipe(observable, mapObs(data => ({ type: 1, data }))), ofValue({ type: 2 }), never());
    const eventStreamSubscription = pipe(eventStream, onNotify(x => subscriber.notify(x)), subscribe(pausableScheduler));
    subscriber
        .add(pausableScheduler)
        .add(modeSubscription)
        .add(eventStreamSubscription);
}));
//# sourceMappingURL=stream.js.map